@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2007 Olivier Teulière
+ * Copyright (C) 2005-2008 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,10 +18,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
+#include <boost/foreach.hpp>
+
 #include <iomanip>
 #include <wctype.h>
 
 #include "freegame.h"
+#include "game_exception.h"
 #include "dic.h"
 #include "tile.h"
 #include "rack.h"
@@ -30,6 +33,10 @@
 #include "pldrack.h"
 #include "results.h"
 #include "player.h"
+#include "player_points_cmd.h"
+#include "player_move_cmd.h"
+#include "player_rack_cmd.h"
+#include "game_move_cmd.h"
 #include "ai_player.h"
 #include "settings.h"
 #include "turn.h"
@@ -60,20 +67,14 @@ int FreeGame::play(const wstring &iCoord, const wstring &iWord)
         Move move(round);
 
         // Update the rack and the score of the current player
-        m_players[m_currPlayer]->endTurn(move, m_history.getSize());
-
-        // Everything is OK, we can play the word
-        helperPlayMove(m_currPlayer, move);
+        recordPlayerMove(move, *m_players[m_currPlayer], true);
     }
     else
     {
         Move move(iWord, iCoord);
 
         // Record the invalid move of the player
-        m_players[m_currPlayer]->endTurn(move, m_history.getSize());
-
-        // Update the game
-        helperPlayMove(m_currPlayer, move);
+        recordPlayerMove(move, *m_players[m_currPlayer], true);
     }
 
     // Next turn
@@ -90,52 +91,78 @@ void FreeGame::playAI(unsigned int p)
 
     AIPlayer *player = static_cast<AIPlayer*>(m_players[p]);
 
-    player->compute(m_dic, m_board, m_history.beforeFirstRound());
-    const Move move = player->getMove();
+    player->compute(getDic(), getBoard(), getHistory().beforeFirstRound());
+    const Move &move = player->getMove();
     if (move.getType() == Move::CHANGE_LETTERS ||
         move.getType() == Move::PASS)
     {
-        ASSERT(checkPass(move.getChangedLetters(), p) == 0, "AI tried to cheat!");
+        ASSERT(checkPass(*player, move.getChangedLetters()) == 0,
+               "AI tried to cheat!");
     }
 
     // Update the rack and the score of the current player
-    player->endTurn(move, m_history.getSize());
+    recordPlayerMove(move, *player, false);
 
-    helperPlayMove(p, move);
     endTurn();
 }
 
 
-int FreeGame::start()
+void FreeGame::recordPlayerMove(const Move &iMove, Player &ioPlayer,
+                                bool isForHuman)
+{
+    Command *pCmd = new PlayerMoveCmd(ioPlayer, iMove);
+    pCmd->setAutoExecution(!isForHuman);
+    accessNavigation().addAndExecute(pCmd);
+}
+
+
+void FreeGame::start()
 {
     ASSERT(getNPlayers(), "Cannot start a game without any player");
 
     // Set the initial racks of the players
     for (unsigned int i = 0; i < getNPlayers(); i++)
     {
-        helperSetRackRandom(i, false, RACK_NEW);
+        const PlayedRack &newRack =
+            helperSetRackRandom(getPlayer(i).getCurrentRack(), false, RACK_NEW);
+        Command *pCmd = new PlayerRackCmd(*m_players[i], newRack);
+        accessNavigation().addAndExecute(pCmd);
     }
 
-    m_currPlayer = 0;
+    firstPlayer();
+
+    accessNavigation().newTurn();
 
     // If the first player is an AI, make it play now
     if (!m_players[m_currPlayer]->isHuman())
     {
         playAI(m_currPlayer);
     }
-
-    return 0;
 }
 
 
 int FreeGame::endTurn()
 {
+    const Move &move = getCurrentPlayer().getLastMove();
+    // Update the game
+    Command *pCmd = new GameMoveCmd(*this, move,
+                                    getCurrentPlayer().getLastRack(),
+                                    m_currPlayer);
+    accessNavigation().addAndExecute(pCmd);
+
     // Complete the rack for the player that just played
-    const Move &move = m_history.getPreviousTurn().getMove();
     if (move.getType() == Move::VALID_ROUND ||
         move.getType() == Move::CHANGE_LETTERS)
     {
-        if (helperSetRackRandom(m_currPlayer, false, RACK_NEW) == 1)
+        try
+        {
+            const PlayedRack &newRack =
+                helperSetRackRandom(getCurrentPlayer().getCurrentRack(), false, RACK_NEW);
+            Command *pCmd = new PlayerRackCmd(*m_players[m_currPlayer],
+                                              newRack);
+            accessNavigation().addAndExecute(pCmd);
+        }
+        catch (EndGameException &e)
         {
             // End of the game
             endGame();
@@ -146,8 +173,10 @@ int FreeGame::endTurn()
     // Next player
     nextPlayer();
 
+    accessNavigation().newTurn();
+
     // If this player is an AI, make it play now
-    if (!m_players[m_currPlayer]->isHuman())
+    if (!getCurrentPlayer().isHuman())
     {
         playAI(m_currPlayer);
     }
@@ -181,11 +210,17 @@ void FreeGame::endGame()
         {
             const PlayedRack &pld = m_players[i]->getCurrentRack();
             pld.getAllTiles(tiles);
-            for (unsigned int j = 0; j < tiles.size(); j++)
+            int points = 0;
+            BOOST_FOREACH(const Tile &tile, tiles)
             {
-                m_players[i]->addPoints(- tiles[j].getPoints());
-                m_players[m_currPlayer]->addPoints(tiles[j].getPoints());
+                points += tile.getPoints();
             }
+            // Add the points to the current player...
+            Command *pCmd = new PlayerPointsCmd(*m_players[m_currPlayer], points);
+            accessNavigation().addAndExecute(pCmd);
+            // ... and remove them from the other player
+            Command *pCmd2 = new PlayerPointsCmd(*m_players[i], -points);
+            accessNavigation().addAndExecute(pCmd2);
         }
     }
 
@@ -194,22 +229,21 @@ void FreeGame::endGame()
 }
 
 
-int FreeGame::checkPass(const wstring &iToChange, unsigned int p) const
+int FreeGame::checkPass(const Player &iPlayer,
+                        const wstring &iToChange) const
 {
-    ASSERT(p < getNPlayers(), "Wrong player number");
-
     // Check that the game is not finished
     if (m_finished)
         return 3;
 
     // Check that the letters are valid for the current dictionary
-    if (!m_dic.validateLetters(iToChange))
+    if (!getDic().validateLetters(iToChange))
         return 4;
 
     // It is forbidden to change letters when the bag does not contain at
     // least 7 letters (this is explicitly stated in the ODS). But it is
     // still allowed to pass
-    Bag bag(m_dic);
+    Bag bag(getDic());
     realBag(bag);
     if (bag.getNbTiles() < 7 && !iToChange.empty())
     {
@@ -217,18 +251,17 @@ int FreeGame::checkPass(const wstring &iToChange, unsigned int p) const
     }
 
     // Check that the letters are all present in the player's rack
-    Player *player = m_players[p];
-    PlayedRack pld = player->getCurrentRack();
+    const PlayedRack &pld = iPlayer.getCurrentRack();
     Rack rack;
     pld.getRack(rack);
-    for (unsigned int i = 0; i < iToChange.size(); i++)
+    BOOST_FOREACH(wchar_t wch, iToChange)
     {
         // Remove the letter from the rack
-        if (!rack.in(Tile(iToChange[i])))
+        if (!rack.in(Tile(wch)))
         {
             return 2;
         }
-        rack.remove(Tile(iToChange[i]));
+        rack.remove(Tile(wch));
     }
 
     // According to the rules in the ODS, it is allowed to pass its turn (no
@@ -244,15 +277,14 @@ int FreeGame::checkPass(const wstring &iToChange, unsigned int p) const
 
 int FreeGame::pass(const wstring &iToChange)
 {
-    int res = checkPass(iToChange, m_currPlayer);
+    Player &player = *m_players[m_currPlayer];
+    int res = checkPass(player, iToChange);
     if (res != 0)
         return res;
 
     Move move(iToChange);
     // End the player's turn
-    m_players[m_currPlayer]->endTurn(move, m_history.getSize());
-    // Update the game
-    helperPlayMove(m_currPlayer, move);
+    recordPlayerMove(move, player, true);
 
     // Next game turn
     endTurn();
@@ -260,9 +292,3 @@ int FreeGame::pass(const wstring &iToChange)
     return 0;
 }
 
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
