@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2009 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -19,6 +19,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
+#include <boost/foreach.hpp>
+#include <sstream>
+
+#include "config.h"
+#if ENABLE_NLS
+#   include <libintl.h>
+#   define _(String) gettext(String)
+#else
+#   define _(String) String
+#endif
+
+
 #include "dic.h"
 #include "tile.h"
 #include "rack.h"
@@ -27,14 +39,13 @@
 #include "results.h"
 #include "player.h"
 #include "game.h"
-#include "game_factory.h"
 #include "turn.h"
 #include "encoding.h"
 #include "game_exception.h"
+#include "turn_cmd.h"
 
 #include "debug.h"
 
-
 const unsigned int Game::RACK_SIZE =  7;
 const int Game::BONUS_POINTS = 50;
 
@@ -50,9 +61,9 @@ Game::Game(const Dictionary &iDic):
 
 Game::~Game()
 {
-    for (unsigned int i = 0; i < getNPlayers(); i++)
+    BOOST_FOREACH(Player *p, m_players)
     {
-        delete m_players[i];
+        delete p;
     }
 }
 
@@ -64,146 +75,6 @@ const Player& Game::getPlayer(unsigned int iNum) const
 }
 
 
-void Game::helperPlayMove(unsigned int iPlayerId, const Move &iMove)
-{
-    // History of the game
-    m_history.setCurrentRack(getPlayer(iPlayerId).getLastRack());
-    m_history.playMove(iPlayerId, m_history.getSize(), iMove);
-
-    // Points
-    m_points += iMove.getScore();
-
-    // For moves corresponding to a valid round, we have much more
-    // work to do...
-    if (iMove.getType() == Move::VALID_ROUND)
-    {
-        helperPlayRound(iPlayerId, iMove.getRound());
-    }
-}
-
-
-
-void Game::helperPlayRound(unsigned int iPlayerId, const Round &iRound)
-{
-    // Copy the round, because we may need to modify it (case of
-    // the joker games).
-    Round round = iRound;
-
-    // Before updating the bag and the board, if we are playing a "joker game",
-    // we replace in the round the joker by the letter it represents
-    // This is currently done by a succession of ugly hacks :-/
-    if (m_variant == kJOKER)
-    {
-        for (unsigned int i = 0; i < round.getWordLen(); i++)
-        {
-            if (round.isPlayedFromRack(i) && round.isJoker(i))
-            {
-                // Is the represented letter still available in the bag?
-                // XXX: this way to get the represented letter sucks...
-                Tile t(towupper(round.getTile(i).toChar()));
-                Bag bag(m_dic);
-                realBag(bag);
-                // FIXME: realBag() does not give us a real bag in this
-                // particular case! This is because Player::endTurn() is called
-                // before Game::helperPlayRound(), which means that the rack
-                // of the player is updated, while the word is not actually
-                // played on the board yet. Since realBag() relies on
-                // Player::getCurrentRack(), it doesn't remove the letters of
-                // the current player, which are in fact available through
-                // Player::getLastRack().
-                // That's why we have to replace the letters of the current
-                // rack and remove the ones from the previous rack...
-                // There is a big design problem here, but i am unsure what is
-                // the best way to fix it.
-                vector<Tile> tiles;
-                getPlayer(iPlayerId).getCurrentRack().getAllTiles(tiles);
-                for (unsigned int j = 0; j < tiles.size(); j++)
-                {
-                    bag.replaceTile(tiles[j]);
-                }
-                getPlayer(iPlayerId).getLastRack().getAllTiles(tiles);
-                for (unsigned int j = 0; j < tiles.size(); j++)
-                {
-                    bag.takeTile(tiles[j]);
-                }
-
-                if (bag.in(t))
-                {
-                    round.setTile(i, t);
-                    // FIXME: This shouldn't be necessary, this is only
-                    // needed because of the stupid way of handling jokers in
-                    // rounds
-                    round.setJoker(i, false);
-                }
-
-                // In a joker game we should have only 1 joker in the rack
-                break;
-            }
-        }
-    }
-
-    // Update the bag
-    // We remove tiles from the bag only when they are played
-    // on the board. When going back in the game, we must only
-    // replace played tiles.
-    // We test a rack when it is set but tiles are left in the bag.
-    for (unsigned int i = 0; i < round.getWordLen(); i++)
-    {
-        if (round.isPlayedFromRack(i))
-        {
-            if (round.isJoker(i))
-            {
-                m_bag.takeTile(Tile::Joker());
-            }
-            else
-            {
-                m_bag.takeTile(round.getTile(i));
-            }
-        }
-    }
-
-    // Update the board
-    m_board.addRound(m_dic, round);
-}
-
-
-int Game::back(unsigned int n)
-{
-    if (m_history.getSize() < n)
-        throw GameException("Cannot go back that far");
-
-    for (unsigned int i = 0; i < n; i++)
-    {
-        prevPlayer();
-        const Move &lastMove = m_history.getPreviousTurn().getMove();
-        // Nothing to cancel if the move was not a valid round
-        if (lastMove.getType() != Move::VALID_ROUND)
-            continue;
-
-        const Round &lastround = lastMove.getRound();
-        // Remove the word from the board, and put its letters back
-        // into the bag
-        m_board.removeRound(m_dic, lastround);
-        for (unsigned int j = 0; j < lastround.getWordLen(); j++)
-        {
-            if (lastround.isPlayedFromRack(j))
-            {
-                if (lastround.isJoker(j))
-                    m_bag.replaceTile(Tile::Joker());
-                else
-                    m_bag.replaceTile(lastround.getTile(j));
-            }
-        }
-        // Remove the points of this round
-        m_points -= lastround.getPoints();
-        // Remove the turns
-        m_players[m_currPlayer]->removeLastTurn();
-        m_history.removeLastTurn();
-    }
-    return 0;
-}
-
-
 void Game::shuffleRack()
 {
     PlayedRack pld = getCurrentPlayer().getCurrentRack();
@@ -214,43 +85,40 @@ void Game::shuffleRack()
 
 void Game::realBag(Bag &ioBag) const
 {
-    vector<Tile> tiles;
-
     // Copy the bag
     ioBag = m_bag;
 
+    vector<Tile> tiles;
+
     // The real content of the bag depends on the game mode
     if (getMode() == kFREEGAME)
     {
         // In freegame mode, take the letters from all the racks
-        for (unsigned int i = 0; i < getNPlayers(); i++)
+        BOOST_FOREACH(const Player *player, m_players)
         {
-            getPlayer(i).getCurrentRack().getAllTiles(tiles);
-            for (unsigned int j = 0; j < tiles.size(); j++)
+            player->getCurrentRack().getAllTiles(tiles);
+            BOOST_FOREACH(const Tile &tile, tiles)
             {
-                ioBag.takeTile(tiles[j]);
+                ioBag.takeTile(tile);
             }
         }
     }
     else
     {
-        /* In training or duplicate mode, take the rack of the current
-         * player only */
+        // In training or duplicate mode, take the rack of the current
+        // player only
         getPlayer(m_currPlayer).getCurrentRack().getAllTiles(tiles);
-        for (unsigned int j = 0; j < tiles.size(); j++)
+        BOOST_FOREACH(const Tile &tile, tiles)
         {
-            ioBag.takeTile(tiles[j]);
+            ioBag.takeTile(tile);
         }
     }
 }
 
 
-int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
+PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
+                                     bool iCheck, set_rack_mode mode) const
 {
-    ASSERT(p < getNPlayers(), "Wrong player number");
-    // FIXME: RACK_MANUAL shouldn't be in the enum
-    ASSERT(mode != RACK_MANUAL, "Invalid rack mode");
-
     // When iCheck is true, we must make sure that there are at least 2 vowels
     // and 2 consonants in the rack up to the 15th turn, and at least one of
     // each starting from the 16th turn.
@@ -266,8 +134,8 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
             min = 1;
     }
 
-    // Make a copy of the current player's rack
-    PlayedRack pld = getPlayer(p).getCurrentRack();
+    // Make a copy of the given rack
+    PlayedRack pld = iPld;
     int nold = pld.getNbOld();
 
     // Create a copy of the bag in which we can do everything we want,
@@ -281,9 +149,9 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
         // letters of the player)
         vector<Tile> tiles;
         pld.getNewTiles(tiles);
-        for (unsigned int i = 0; i < tiles.size(); i++)
+        BOOST_FOREACH(const Tile &tile, tiles)
         {
-            bag.replaceTile(tiles[i]);
+            bag.replaceTile(tile);
         }
         pld.resetNew();
     }
@@ -292,9 +160,9 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
         // Replace all the tiles in the bag before choosing random ones
         vector<Tile> tiles;
         pld.getAllTiles(tiles);
-        for (unsigned int i = 0; i < tiles.size(); i++)
+        BOOST_FOREACH(const Tile &tile, tiles)
         {
-            bag.replaceTile(tiles[i]);
+            bag.replaceTile(tile);
         }
         // RACK_NEW with an empty rack is equivalent to RACK_ALL
         pld.reset();
@@ -303,24 +171,26 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
     }
     else
     {
-        throw GameException("Not a random mode");
+        throw GameException(_("Not a random mode"));
     }
 
     // Get the tiles remaining on the rack
     vector<Tile> tiles;
     pld.getOldTiles(tiles);
-    ASSERT(tiles.size() < RACK_SIZE,
-           "Cannot complete the rack, it is already complete");
+    // The rack is already complete, there is nothing to do
+    // TODO: add a log here
+    if (tiles.size() >= RACK_SIZE)
+        return iPld;
 
     bool jokerAdded = false;
     // Are we dealing with a normal game or a joker game?
-    if (m_variant == kJOKER)
+    if (m_variant == kJOKER || m_variant == kEXPLOSIVE)
     {
         // 1) Is there already a joker in the remaining letters of the rack?
         bool jokerFound = false;
-        for (unsigned int i = 0; i < tiles.size(); i++)
+        BOOST_FOREACH(const Tile &tile, tiles)
         {
-            if (tiles[i].isJoker())
+            if (tile.isJoker())
             {
                 jokerFound = true;
                 break;
@@ -336,7 +206,7 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
         }
 
         // 3) Remove all the jokers from the bag, to avoid taking another one
-        for (unsigned int i = 0; i < bag.in(Tile::Joker()); ++i)
+        while (bag.in(Tile::Joker()))
         {
             bag.takeTile(Tile::Joker());
         }
@@ -346,34 +216,35 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
     // (i.e. minimum required, minus what we already have in the rack)
     unsigned int neededVowels = min;
     unsigned int neededConsonants = min;
-    for (unsigned int i = 0; i < tiles.size(); ++i)
+    BOOST_FOREACH(const Tile &tile, tiles)
     {
-        if (neededVowels > 0 && tiles[i].isVowel())
+        if (neededVowels > 0 && tile.isVowel())
             neededVowels--;
-        if (neededConsonants > 0 && tiles[i].isConsonant())
+        if (neededConsonants > 0 && tile.isConsonant())
             neededConsonants--;
     }
 
     // Nothing in the rack, nothing in the bag --> end of the (free)game
     if (bag.getNbTiles() == 0 && pld.getNbTiles() == 0)
     {
-        return 1;
+        throw EndGameException(_("The bag is empty"));
     }
 
     // Check whether it is possible to complete the rack properly
     if (bag.getNbVowels() < neededVowels ||
         bag.getNbConsonants() < neededConsonants)
     {
-        return 1;
+        throw EndGameException(_("Not enough vowels or consonants to complete the rack"));
     }
     // End of game condition
     if (iCheck)
     {
+        // FIXME: redundant checks?
         if (bag.getNbVowels() < neededVowels ||
             bag.getNbConsonants() < neededConsonants ||
             (bag.getNbTiles() + tiles.size()) == 1)
         {
-            return 1;
+            throw EndGameException(_("Not enough vowels or consonants to complete the rack"));
         }
     }
 
@@ -386,7 +257,7 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
     // requirements will be met.
     while (bag.getNbTiles() != 0 && pld.getNbTiles() < RACK_SIZE)
     {
-        Tile l = bag.selectRandom();
+        const Tile &l = bag.selectRandom();
         bag.takeTile(l);
         pld.addNew(l);
     }
@@ -396,9 +267,9 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
         // Bad luck... we have to reject the rack
         vector<Tile> rejectedTiles;
         pld.getAllTiles(rejectedTiles);
-        for (unsigned int i = 0; i < rejectedTiles.size(); i++)
+        BOOST_FOREACH(const Tile &rejTile, rejectedTiles)
         {
-            bag.replaceTile(rejectedTiles[i]);
+            bag.replaceTile(rejTile);
         }
         pld.reset();
         // Do not mark the rack as rejected if it was empty
@@ -420,13 +291,13 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
             // Actually this should never happen, but it doesn't hurt to check...
             // FIXME: this test is not completely right, because it supposes no
             // letter can be at the same time a vowel and a consonant
-            return 3;
+            throw EndGameException("Not enough vowels or consonants to complete the rack");
         }
 
         // Get the required vowels and consonants first
         for (unsigned int i = 0; i < neededVowels; ++i)
         {
-            Tile l = bag.selectRandomVowel();
+            const Tile &l = bag.selectRandomVowel();
             bag.takeTile(l);
             pld.addNew(l);
             // Handle the case where the vowel can also be considered
@@ -436,7 +307,7 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
         }
         for (unsigned int i = 0; i < neededConsonants; ++i)
         {
-            Tile l = bag.selectRandomConsonant();
+            const Tile &l = bag.selectRandomConsonant();
             bag.takeTile(l);
             pld.addNew(l);
         }
@@ -447,12 +318,69 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
         // Now complete the rack with truly random letters
         while (bag.getNbTiles() != 0 && pld.getNbTiles() < RACK_SIZE)
         {
-            Tile l = bag.selectRandom();
+            const Tile &l = bag.selectRandom();
             bag.takeTile(l);
             pld.addNew(l);
         }
     }
 
+    // In explosive games, we have to perform a search, then replace the
+    // joker with the letter providing the best score
+    // A joker coming from a previous rack is not replaced
+    if (m_variant == kEXPLOSIVE && jokerAdded)
+    {
+        Rack rack;
+        pld.getRack(rack);
+
+        BestResults res;
+        res.search(getDic(), getBoard(), rack,  getHistory().beforeFirstRound());
+        if (res.size())
+        {
+            PlayedRack pldCopy = pld;
+
+            // Get the best word
+            const Round & bestRound = res.get(0);
+#ifdef DEBUG
+                    cout << "helperSetRackRandom(): initial rack: "
+                         << convertToMb(pld.toString()) << " (best word: "
+                         << convertToMb(bestRound.getWord()) << ")" << endl;
+#endif
+            // Identify the joker
+            for (unsigned int i = 0; i < bestRound.getWordLen(); ++i)
+            {
+                if (bestRound.isJoker(i) && bestRound.isPlayedFromRack(i))
+                {
+                    const Tile &jokerTile = bestRound.getTile(i);
+                    Tile replacingTile(towupper(jokerTile.toChar()));
+#ifdef DEBUG
+                    cout << "helperSetRackRandom(): replacing Joker with "
+                         << convertToMb(replacingTile.toChar()) << endl;
+#endif
+                    // If the bag does not contain this letter anymore,
+                    // simply keep the joker in the rack.
+                    if (bag.in(replacingTile))
+                    {
+                        // The bag contains the replacing letter
+                        // We need to swap the joker (it is necessarily in the
+                        // new tiles, because jokerAdded is true)
+                        Rack tmpRack;
+                        pld.getNew(tmpRack);
+                        ASSERT(tmpRack.in(Tile::Joker()),
+                               "No joker found in the new tiles!");
+                        tmpRack.remove(Tile::Joker());
+                        tmpRack.add(replacingTile);
+                        pld.setNew(tmpRack);
+
+                        // Make sure the invariant is still correct, otherwise we keep the joker
+                        if (!pld.checkRack(min, min))
+                            pld = pldCopy;
+                    }
+                    break;
+                }
+            }
+        }
+    }
+
     // Shuffle the new tiles, to hide the order we imposed (joker first in a
     // joker game, then needed vowels, then needed consonants, and rest of the
     // rack)
@@ -461,33 +389,25 @@ int Game::helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode)
     // Post-condition check. This should never fail, of course :)
     ASSERT(pld.checkRack(min, min), "helperSetRackRandom() is buggy!");
 
-    // Until now we didn't modify anything except local variables.
-    // Let's "commit" the changes
-    m_players[p]->setCurrentRack(pld);
-
-    return 0;
+    return pld;
 }
 
 
 bool Game::rackInBag(const Rack &iRack, const Bag &iBag) const
 {
-    const vector<Tile>& allTiles = m_dic.getAllTiles();
-    vector<Tile>::const_iterator it;
-    for (it = allTiles.begin(); it != allTiles.end(); it++)
+    BOOST_FOREACH(const Tile &t, m_dic.getAllTiles())
     {
-        if (iRack.in(*it) > iBag.in(*it))
+        if (iRack.in(t) > iBag.in(t))
             return false;
     }
     return true;
 }
 
 
-int Game::helperSetRackManual(unsigned int p, bool iCheck, const wstring &iLetters)
+PlayedRack Game::helperSetRackManual(bool iCheck, const wstring &iLetters) const
 {
-    ASSERT(p < getNPlayers(), "Wrong player number");
-
     if (!m_dic.validateLetters(iLetters, L"+-"))
-        return 3;
+        throw GameException(_("Some letters are invalid for the current dictionary"));
 
     PlayedRack pld;
     pld.setManual(iLetters);
@@ -496,8 +416,7 @@ int Game::helperSetRackManual(unsigned int p, bool iCheck, const wstring &iLette
     pld.getRack(rack);
     if (!rackInBag(rack, m_bag))
     {
-        pld.reset();
-        return 1;
+        throw GameException(_("The bag does not contain all these letters"));
     }
 
     if (iCheck)
@@ -509,12 +428,11 @@ int Game::helperSetRackManual(unsigned int p, bool iCheck, const wstring &iLette
         else
             min = 1;
         if (!pld.checkRack(min, min))
-            return 2;
+        {
+            throw GameException(_("Not enough vowels or consonants in this rack"));
+        }
     }
-
-    m_players[p]->setCurrentRack(pld);
-
-    return 0;
+    return pld;
 }
 
 /*********************************************************
@@ -524,8 +442,10 @@ int Game::helperSetRackManual(unsigned int p, bool iCheck, const wstring &iLette
 unsigned int Game::getNHumanPlayers() const
 {
     unsigned int count = 0;
-    for (unsigned int i = 0; i < getNPlayers(); i++)
-        count += (getPlayer(i).isHuman() ? 1 : 0);
+    BOOST_FOREACH(const Player *player, m_players)
+    {
+        count += (player->isHuman() ? 1 : 0);
+    }
     return count;
 }
 
@@ -540,14 +460,29 @@ void Game::addPlayer(Player *iPlayer)
 }
 
 
+void Game::firstPlayer()
+{
+    ASSERT(getNPlayers() != 0, "Expected at least one player");
+    // Make sure there is something to do
+    if (m_currPlayer == 0)
+        return;
+
+    Command *pCmd = new CurrentPlayerCmd(*this, 0);
+    accessNavigation().addAndExecute(pCmd);
+}
+
+
 void Game::prevPlayer()
 {
     ASSERT(getNPlayers() != 0, "Expected at least one player");
 
+    unsigned int newPlayerId;
     if (m_currPlayer == 0)
-        m_currPlayer = getNPlayers() - 1;
+        newPlayerId = getNPlayers() - 1;
     else
-        m_currPlayer--;
+        newPlayerId = m_currPlayer - 1;
+    Command *pCmd = new CurrentPlayerCmd(*this, newPlayerId);
+    accessNavigation().addAndExecute(pCmd);
 }
 
 
@@ -555,15 +490,18 @@ void Game::nextPlayer()
 {
     ASSERT(getNPlayers() != 0, "Expected at least one player");
 
+    unsigned int newPlayerId;
     if (m_currPlayer == getNPlayers() - 1)
-        m_currPlayer = 0;
+        newPlayerId = 0;
     else
-        m_currPlayer++;
+        newPlayerId = m_currPlayer + 1;
+    Command *pCmd = new CurrentPlayerCmd(*this, newPlayerId);
+    accessNavigation().addAndExecute(pCmd);
 }
 
 
 int Game::checkPlayedWord(const wstring &iCoord,
-                          const wstring &iWord, Round &oRound)
+                          const wstring &iWord, Round &oRound) const
 {
     ASSERT(getNPlayers() != 0, "Expected at least one player");
 
@@ -634,12 +572,36 @@ int Game::checkPlayedWord(const wstring &iCoord,
     return 0;
 }
 
-/****************************************************************/
-/****************************************************************/
 
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
+Game::CurrentPlayerCmd::CurrentPlayerCmd(Game &ioGame,
+                             unsigned int iPlayerId)
+    : m_game(ioGame), m_newPlayerId(iPlayerId), m_oldPlayerId(0)
+{
+}
+
+
+void Game::CurrentPlayerCmd::doExecute()
+{
+    m_oldPlayerId = m_game.currPlayer();
+    m_game.setCurrentPlayer(m_newPlayerId);
+}
+
+
+void Game::CurrentPlayerCmd::doUndo()
+{
+    m_game.setCurrentPlayer(m_oldPlayerId);
+}
+
+
+wstring Game::CurrentPlayerCmd::toString() const
+{
+    wostringstream oss;
+    oss << L"CurrentPlayerCmd (new player: " << m_newPlayerId;
+    if (isExecuted())
+    {
+        oss << L"  old player: " << m_oldPlayerId;
+    }
+    oss << L")";
+    return oss.str();
+}
+
