@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 1999-2009 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -28,6 +28,8 @@
 #include "bag.h"
 #include "board.h"
 #include "history.h"
+#include "navigation.h"
+#include "command.h"
 
 class Dictionary;
 class Player;
@@ -73,7 +75,8 @@ public:
     enum GameVariant
     {
         kNONE,      // Normal game rules
-        kJOKER      // Joker game
+        kJOKER,     // Joker game
+        kEXPLOSIVE  // "Explosive" game
     };
 
     /**
@@ -93,14 +96,28 @@ public:
      * You should never create a new dictionary object while a Game
      * object still exists
      */
-    const Dictionary & getDic() const   { return m_dic; }
+    const Dictionary & getDic() const { return m_dic; }
 
     /// Get the board
     const Board& getBoard() const { return m_board; }
+    Board & accessBoard() { return m_board; }
     /// Get the bag
     const Bag& getBag() const { return m_bag; }
+    Bag & accessBag() { return m_bag; }
+    /**
+     * The realBag is the current bag minus all the racks
+     * present in the game. It represents the actual
+     * letters that are left in the bag.
+     * FIXME: in Duplicate mode, this method uses m_currPlayer to find the
+     * rack of the player. Since not all the players played the same word,
+     * it is important to set m_currPlayer accurately before!
+     */
+    void realBag(Bag &iBag) const;
+
+
     /// Get the history of the game */
     const History& getHistory() const { return m_history; }
+    History & accessHistory() { return m_history; }
 
     /***************
      * Methods to access players.
@@ -127,7 +144,7 @@ public:
      * AI players are handled automatically, so if the game only has AI
      * players, it will play until the end.
      */
-    virtual int start() = 0;
+    virtual void start() = 0;
 
     /**
      * Method used by human players to play the word iWord at coordinates
@@ -149,15 +166,6 @@ public:
      */
     virtual int play(const wstring &iCoord, const wstring &iWord) = 0;
 
-    /**
-     * Go back to turn iTurn.
-     * We must have: iTurn < getHistory().getSize()
-     * Possible return values:
-     *  0: everything went fine
-     *  1: iTurn is invalid
-     */
-    int back(unsigned int iTurn);
-
     /// Shuffle the rack of the current player
     void shuffleRack();
 
@@ -199,37 +207,63 @@ public:
      * Setting the rack
      ***************/
 
-    enum set_rack_mode {RACK_ALL, RACK_NEW, RACK_MANUAL};
+    enum set_rack_mode {RACK_ALL, RACK_NEW};
 
+    void addPoints(int iPoints) { m_points += iPoints; }
 
-protected:
-    /// All the players, indexed by their ID
-    vector<Player*> m_players;
-    /// ID of the "current" player
-    unsigned int m_currPlayer;
-
-// TODO: check what should be private and what should be protected
-// private:
+    const Navigation & getNavigation() const { return m_navigation; }
+    Navigation & accessNavigation() { return m_navigation; }
 
+private:
     /// Variant
     GameVariant m_variant;
 
     /// Dictionary currently associated to the game
     const Dictionary & m_dic;
 
-    /// Bag
-    Bag m_bag;
+    /**
+     * History of the game.
+     */
+    History m_history;
+
+    Navigation m_navigation;
+
+    int m_points;
+
+    /// Change the player who is supposed to play
+    void setCurrentPlayer(unsigned int iPlayerId) { m_currPlayer = iPlayerId; }
+
+    /// Command used to keep track of the current player changes
+    class CurrentPlayerCmd: public Command
+    {
+        public:
+            CurrentPlayerCmd(Game &ioGame,
+                             unsigned int iPlayerId);
+
+            virtual wstring toString() const;
+
+        protected:
+            virtual void doExecute();
+            virtual void doUndo();
+
+        private:
+            Game &m_game;
+            unsigned int m_newPlayerId;
+            unsigned int m_oldPlayerId;
+    };
+
+// TODO: check what should be private and what should be protected
+protected:
+    /// All the players, indexed by their ID
+    vector<Player*> m_players;
+    /// ID of the "current" player
+    unsigned int m_currPlayer;
 
     /// Board
     Board m_board;
 
-    /**
-     * History of the game.
-     * The vector is indexed by the number of turns in the game
-     */
-    History m_history;
-
-    int m_points;
+    /// Bag
+    Bag m_bag;
 
     bool m_finished;
 
@@ -237,16 +271,8 @@ protected:
      * Helper functions
      *********************************************************/
 
-    /** Play a Move for the given player, updating game history */
-    void helperPlayMove(unsigned int iPlayerId, const Move &iMove);
-
     /**
-     * Set the rack randomly for the player p
-     * Possible return values:
-     *  0: everything went fine
-     *  1: the game is over
-     *  3: there is no chance to set the rack with the vowels/consonants
-     *     constraints
+     * Complete the given rack randomly.
      *
      * Completing a rack randomly is more complex than it seems, because we
      * must take into account several constraints:
@@ -266,17 +292,16 @@ protected:
      *    This also means we have to check whether completing the rack with the
      *    requirements is possible...
      */
-    int helperSetRackRandom(unsigned int p, bool iCheck, set_rack_mode mode);
+    PlayedRack helperSetRackRandom(const PlayedRack &iPld,
+                                   bool iCheck, set_rack_mode mode) const;
 
     /**
-     * Set the rack for the player p with the given letters
-     * Possible return values:
-     *  0: everything went fine
-     *  1: the bag doesn't have the wanted letters
-     *  2: the rack was checked for vowels/consonants and was not correct
+     * Return a rack for the given letters, after performing some checks.
+     * The '+' and '-' signs are accepted in the letters but ignored.
      */
-    int helperSetRackManual(unsigned int p, bool iCheck, const wstring &iLetters);
+    PlayedRack helperSetRackManual(bool iCheck, const wstring &iLetters) const;
 
+    void firstPlayer();
     void prevPlayer();
     void nextPlayer();
 
@@ -292,24 +317,14 @@ protected:
      */
     bool rackInBag(const Rack &iRack, const Bag &iBag) const;
 
-    /**
-     * The realBag is the current bag minus all the racks
-     * present in the game. It represents the actual
-     * letters that are left in the bag.
-     * FIXME: in Duplicate mode, this method uses m_currPlayer to find the
-     * rack of the player. Since not all the players played the same word,
-     * it is important to set m_currPlayer accurately before!
-     */
-    void realBag(Bag &iBag) const;
-
     /**
      * This function checks whether it is legal to play the given word at the
      * given coordinates. If so, the function fills a Round object, also given
      * as a parameter.
      * Possible return values: same as the play() method
      */
-    int  checkPlayedWord(const wstring &iCoord,
-                         const wstring &iWord, Round &oRound);
+    int checkPlayedWord(const wstring &iCoord,
+                        const wstring &iWord, Round &oRound) const;
 
     /**
      * load games from File using the first format.
@@ -319,7 +334,7 @@ protected:
 
     /**
      * load games from File using advanced format (since Eliot 1.5)
-     * This format is used for Duplicate, Freegame, ...
+     * This format is used for Duplicate, FreeGame, ...
      */
     static Game* gameLoadFormat_15(FILE *fin, const Dictionary& iDic);
 
@@ -333,21 +348,7 @@ protected:
      */
     void gameSaveFormat_15(ostream &out) const;
 
-private:
-
-    /**
-     * Play a round on the board.
-     * This should only be called by helperPlayMove().
-     */
-    void helperPlayRound(unsigned int iPlayerId, const Round &iRound);
-
 };
 
 #endif /* _GAME_H_ */
 
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
