@@ -35,6 +35,7 @@
 #include "duplicate.h"
 #include "encoding.h"
 #include "game_exception.h"
+#include "game_move_cmd.h"
 
 using namespace std;
 
@@ -124,8 +125,7 @@ Game* Game::gameLoadFormat_14(FILE *fin, const Dictionary& iDic)
 
             /* rack */
             strncpy(rack, token, sizeof(rack));
-            static_cast<Training*>(pGame)->setRack(RACK_MANUAL, false,
-                                                   convertToWc(rack));
+            static_cast<Training*>(pGame)->setRackManual(false, convertToWc(rack));
 
             /* word */
             token = strtok(NULL, delim);
@@ -390,7 +390,10 @@ Game* Game::gameLoadFormat_15(FILE *fin, const Dictionary& iDic)
 //                     pGame->m_players[player]->endTurn(round,num - 1);
 
         // Play the round
-        pGame->helperPlayRound(pGame->m_currPlayer, round);
+        GameMoveCmd cmd(*pGame, Move(round),
+                        pGame->getCurrentPlayer().getLastRack(),
+                        pGame->m_currPlayer);
+        cmd.execute();
     }
 
     /**************************************/
@@ -601,9 +604,3 @@ void Game::gameSaveFormat_15(ostream &out) const
     }
 }
 
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
