@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2009 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,10 +21,12 @@
 #include <math.h>
 #include <QtGui/QPainter>
 #include <QtGui/QPaintEvent>
+#include <QtCore/QSettings>
 
 #include "board_widget.h"
+#include "prefs_dialog.h"
 #include "qtcommon.h"
-#include "game.h"
+#include "public_game.h"
 #include "tile.h"
 #include "board.h"
 
@@ -53,7 +55,7 @@ BoardWidget::BoardWidget(QWidget *parent)
 }
 
 
-void BoardWidget::setGame(const Game *iGame)
+void BoardWidget::setGame(const PublicGame *iGame)
 {
     m_game = iGame;
     refresh();
@@ -71,16 +73,22 @@ QSize BoardWidget::sizeHint() const
     return QSize(400, 400);
 }
 
-
 void BoardWidget::paintEvent(QPaintEvent *)
 {
-    int size = std::min(width(), height());
-    int squareSize = (int)floor((size - 1) / (BOARD_MAX - BOARD_MIN + 2));
+    const int size = std::min(width(), height());
+    const int squareSize = (int)floor((size - 1) / (BOARD_MAX - BOARD_MIN + 2));
 
     // The font must grow with the square size
-    QFont f = font();
-    f.setPixelSize(squareSize * 2 / 3);
-    setFont(f);
+    QFont letterFont = font();
+    letterFont.setPixelSize(squareSize * 2 / 3);
+
+    QFont pointsFont = font();
+    const double pointsCoeff = 8. / 25.;
+    pointsFont.setPixelSize(squareSize * pointsCoeff);
+
+    // Should we display the tiles points?
+    QSettings qs(ORGANIZATION, PACKAGE_NAME);
+    bool showPoints = qs.value(PrefsDialog::kINTF_SHOW_TILES_POINTS, true).toBool();
 
     // XXX: Naive implementation: we repaint everything every time
     QPainter painter(this);
@@ -89,6 +97,9 @@ void BoardWidget::paintEvent(QPaintEvent *)
     {
         for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
         {
+            const unsigned int xPos = (col - BOARD_MIN + 1) * squareSize;
+            const unsigned int yPos = (row - BOARD_MIN + 1) * squareSize;
+
             // Set the brush color
             if (m_game != NULL && !m_game->getBoard().getTile(row, col).isEmpty())
             {
@@ -107,9 +118,7 @@ void BoardWidget::paintEvent(QPaintEvent *)
                 painter.setBrush(L2Colour);
             else
                 painter.setBrush(EmptyColour);
-            painter.drawRect((col - BOARD_MIN + 1) * squareSize,
-                             (row - BOARD_MIN + 1) * squareSize,
-                             squareSize, squareSize);
+            painter.drawRect(xPos, yPos, squareSize, squareSize);
 
             // Draw the letter
             if (m_game != NULL && !m_game->getBoard().getTile(row, col).isEmpty())
@@ -117,16 +126,27 @@ void BoardWidget::paintEvent(QPaintEvent *)
                 wchar_t chr = m_game->getBoard().getTile(row, col).toChar();
                 if (m_game->getBoard().getCharAttr(row, col) & ATTR_JOKER)
                     painter.setPen(JokerColour);
-                painter.drawText((col - BOARD_MIN + 1) * squareSize,
-                                 (row - BOARD_MIN + 1) * squareSize + 1,
-                                 squareSize, squareSize,
-                                 Qt::AlignCenter,
-                                 qfw(wstring(1, chr)));
+                painter.setFont(letterFont);
+                painter.drawText(xPos, yPos + 1, squareSize, squareSize,
+                                 Qt::AlignCenter, qfw(wstring(1, chr)));
                 painter.setPen(NormalColour);
+
+                // Draw the points of the tile
+                if (showPoints &&
+                    !m_game->getBoard().getCharAttr(row, col) & ATTR_JOKER)
+                {
+                    painter.setFont(pointsFont);
+                    painter.drawText(xPos + squareSize * (1 - pointsCoeff),
+                                     yPos + squareSize * (1 - pointsCoeff) + 1,
+                                     squareSize * pointsCoeff, squareSize * pointsCoeff + 3,
+                                     Qt::AlignRight | Qt::AlignBottom,
+                                     QString("%1").arg(m_game->getBoard().getTile(row, col).getPoints()));
+                }
             }
         }
     }
     // Draw the coordinates
+    painter.setFont(letterFont);
     for (unsigned x = 1; x <= BOARD_MAX - BOARD_MIN + 1; ++x)
     {
         painter.drawText(x * squareSize, 1,
