@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2009 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,11 +34,12 @@
 
 #include "main_window.h"
 #include "dic.h"
+#include "dic_exception.h"
 #include "encoding.h"
 #include "header.h"
 #include "game_factory.h"
 #include "game.h"
-#include "freegame.h"
+#include "public_game.h"
 #include "player.h"
 #include "history.h"
 #include "turn.h"
@@ -52,7 +53,6 @@
 #include "player_widget.h"
 #include "history_widget.h"
 #include "dic_tools_widget.h"
-#include "training_widget.h"
 #include "aux_window.h"
 #include "qtcommon.h"
 
@@ -84,8 +84,11 @@ MainWindow::MainWindow(QWidget *iParent)
     cout << "Rand seed: " << val << endl;
 #endif
 
-    QObject::connect(this, SIGNAL(gameChanged(const Game*)),
-                     this, SLOT(updateForGame(const Game*)));
+    QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                     this, SLOT(updateForGame(const PublicGame*)));
+    QObject::connect(this, SIGNAL(gameUpdated()),
+                     this, SLOT(refresh()));
+    refresh();
 
     // Status bar
     statusBar()->addWidget(new QLabel, 1);
@@ -97,8 +100,8 @@ MainWindow::MainWindow(QWidget *iParent)
 
     // Board
     BoardWidget *boardWidget = new BoardWidget;
-    QObject::connect(this, SIGNAL(gameChanged(const Game*)),
-                     boardWidget, SLOT(setGame(const Game*)));
+    QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                     boardWidget, SLOT(setGame(const PublicGame*)));
     QObject::connect(this, SIGNAL(gameUpdated()),
                      boardWidget, SLOT(refresh()));
 
@@ -115,32 +118,38 @@ MainWindow::MainWindow(QWidget *iParent)
 
     m_ui.groupBoxTest->setLayout(hlayout);
 
+#if 1
     // History
     HistoryTabWidget *historyTab = new HistoryTabWidget;
-    QObject::connect(this, SIGNAL(gameChanged(const Game*)),
-                     historyTab, SLOT(setGame(const Game*)));
+    QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                     historyTab, SLOT(setGame(const PublicGame*)));
     QObject::connect(this, SIGNAL(gameUpdated()),
                      historyTab, SLOT(refresh()));
     QHBoxLayout *hlayout2 = new QHBoxLayout;
     hlayout2->addWidget(historyTab);
     m_ui.groupBoxHistory->setLayout(hlayout2);
+#else
+    m_ui.groupBoxHistory->hide();
+#endif
 
     // Players racks
     m_ui.groupBoxPlayers->hide();
-    PlayerTabWidget *players = new PlayerTabWidget(NULL);
-    m_ui.groupBoxPlayers->layout()->addWidget(players);
-    QObject::connect(this, SIGNAL(gameChangedNonConst(Game*)),
-                     players, SLOT(setGame(Game*)));
-    QObject::connect(this, SIGNAL(gameUpdated()), players, SLOT(refresh()));
+    m_playersWidget = new PlayerTabWidget(NULL);
+    m_ui.groupBoxPlayers->layout()->addWidget(m_playersWidget);
+    QObject::connect(this, SIGNAL(gameChangedNonConst(PublicGame*)),
+                     m_playersWidget, SLOT(setGame(PublicGame*)));
+    QObject::connect(this, SIGNAL(gameUpdated()), m_playersWidget, SLOT(refresh()));
 
-    QObject::connect(players, SIGNAL(gameUpdated()), this, SIGNAL(gameUpdated()));
-    QObject::connect(players, SIGNAL(notifyProblem(QString)),
+    QObject::connect(m_playersWidget, SIGNAL(gameUpdated()), this, SIGNAL(gameUpdated()));
+    QObject::connect(m_playersWidget, SIGNAL(notifyProblem(QString)),
                      this, SLOT(displayErrorMsg(QString)));
+    QObject::connect(m_playersWidget, SIGNAL(notifyInfo(QString)),
+                     this, SLOT(displayInfoMsg(QString)));
 
     // Players score
     ScoreWidget *scores = new ScoreWidget;
-    QObject::connect(this, SIGNAL(gameChanged(const Game*)),
-                     scores, SLOT(setGame(const Game*)));
+    QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                     scores, SLOT(setGame(const PublicGame*)));
     QObject::connect(this, SIGNAL(gameUpdated()),
                      scores, SLOT(refresh()));
     m_ui.groupBoxPlayers->layout()->addWidget(scores);
@@ -157,9 +166,10 @@ MainWindow::MainWindow(QWidget *iParent)
         {
             m_dic = new Dictionary(qtl(dicPath));
         }
-        catch (...)
+        catch (DicException &e)
         {
-            displayErrorMsg(_q("Cannot load dictionary '%1' indicated in the preferences").arg(dicPath));
+            displayErrorMsg(_q("Cannot load dictionary '%1' indicated in the "
+                               "preferences.\nReason: %2").arg(dicPath).arg(e.what()));
         }
     }
     emit dicChanged(m_dic);
@@ -194,23 +204,71 @@ void MainWindow::destroyCurrentGame()
 }
 
 
-void MainWindow::updateForGame(const Game *iGame)
+void MainWindow::refresh()
+{
+    if (m_game != NULL)
+    {
+        bool isFirstTurn = m_game->isFirstTurn();
+        bool isLastTurn = m_game->isLastTurn();
+        m_actionHistoryFirstTurn->setEnabled(!isFirstTurn);
+        m_actionHistoryPrevTurn->setEnabled(!isFirstTurn);
+        m_actionHistoryNextTurn->setEnabled(!isLastTurn);
+        m_actionHistoryLastTurn->setEnabled(!isLastTurn);
+        m_actionHistoryReplayTurn->setEnabled(!isLastTurn);
+#ifdef DEBUG
+        cout << endl << endl;
+        m_game->printTurns();
+#endif
+    }
+}
+
+
+void MainWindow::prefsUpdated()
+{
+    // Disconnect the training rack updates from the "Plus 1" tab of the
+    // dictionary tools
+    m_playersWidget->disconnect(SIGNAL(trainingRackUpdated(const QString&)));
+    // Reconnect it only if needed
+    if (m_dicToolsWindow != NULL)
+    {
+        QSettings qs(ORGANIZATION, PACKAGE_NAME);
+        if (qs.value(PrefsDialog::kINTF_LINK_TRAINING_7P1, false).toBool())
+        {
+            QObject::connect(m_playersWidget,
+                             SIGNAL(trainingRackUpdated(const QString&)),
+                             &m_dicToolsWindow->getWidget(),
+                             SLOT(setPlus1Rack(const QString&)));
+        }
+    }
+
+    // Probably useless in most cases (currently only used for
+    // the History alignment)
+    emit gameUpdated();
+}
+
+
+void MainWindow::updateForGame(const PublicGame *iGame)
 {
     if (iGame == NULL)
     {
         m_actionGameSaveAs->setEnabled(false);
         m_actionGamePrint->setEnabled(false);
+        m_actionHistoryFirstTurn->setEnabled(false);
+        m_actionHistoryPrevTurn->setEnabled(false);
+        m_actionHistoryNextTurn->setEnabled(false);
+        m_actionHistoryLastTurn->setEnabled(false);
+        m_actionHistoryReplayTurn->setEnabled(false);
         setWindowTitle(_q("No game") + " - Eliot");
     }
     else
     {
         m_actionGamePrint->setEnabled(true);
-        if (iGame->getMode() == Game::kTRAINING)
+        if (iGame->getMode() == PublicGame::kTRAINING)
         {
             m_actionGameSaveAs->setEnabled(true);
             setWindowTitle(_q("Training mode") + " - Eliot");
         }
-        else if (iGame->getMode() == Game::kDUPLICATE)
+        else if (iGame->getMode() == PublicGame::kDUPLICATE)
         {
             m_actionGameSaveAs->setEnabled(false);
             setWindowTitle(_q("Duplicate game") + " - Eliot");
@@ -229,21 +287,9 @@ void MainWindow::updateStatusBar(const Dictionary *iDic)
     if (iDic == NULL)
         m_dicNameLabel->setText("No dictionary");
     else {
-        if (iDic->getHeader().getVersion() != 0)
-        {
-            QString dicName = qfw(m_dic->getHeader().getName());
-            m_dicNameLabel->setText(_q("Dictionary: %1").arg(dicName));
-            m_dicNameLabel->setToolTip("");
-        }
-        else
-        {
-            m_dicNameLabel->setText(_q("Dictionary: Unknown (old format)"));
-            QString warning = _q("The dictionary name cannot be "
-                    "retrieved, because you are using an old dictionary format.\n"
-                    "You can probably download a newer version of the dictionary "
-                    "on http://www.nongnu.org/eliot/");
-            m_dicNameLabel->setToolTip(warning);
-        }
+        QString dicName = qfw(m_dic->getHeader().getName());
+        m_dicNameLabel->setText(_q("Dictionary: %1").arg(dicName));
+        m_dicNameLabel->setToolTip("");
     }
 }
 
@@ -285,6 +331,11 @@ void MainWindow::writeSettings() const
     settings.beginGroup(m_windowName);
     settings.setValue("size", size());
     settings.setValue("pos", pos());
+    // Do not save this splitter when no game is present, because the players
+    // group is hidden and the splitter is not in its normal place
+    if (m_game)
+        settings.setValue("splitterHoriz", m_ui.splitterHoriz->saveState());
+    settings.setValue("splitterVert", m_ui.splitterVert->saveState());
     settings.endGroup();
 }
 
@@ -297,6 +348,8 @@ void MainWindow::readSettings()
     if (size.isValid())
         resize(size);
     move(settings.value("pos", QPoint(200, 200)).toPoint());
+    m_ui.splitterHoriz->restoreState(settings.value("splitterHoriz").toByteArray());
+    m_ui.splitterVert->restoreState(settings.value("splitterVert").toByteArray());
     settings.endGroup();
 }
 
@@ -304,12 +357,13 @@ void MainWindow::readSettings()
 QAction * MainWindow::addMenuAction(QMenu *menu, QString iText,
                                     const QKeySequence &iShortcut,
                                     QString iStatusTip, const char *iMember,
-                                    bool iCheckable)
+                                    bool iCheckable, QIcon icon)
 {
     QAction *action = new QAction(iText, this);
     action->setShortcut(iShortcut);
     action->setStatusTip(iStatusTip);
     action->setCheckable(iCheckable);
+    action->setIcon(icon);
     QObject::connect(action, SIGNAL(triggered()), this, iMember);
     menu->addAction(action);
     return action;
@@ -318,6 +372,11 @@ QAction * MainWindow::addMenuAction(QMenu *menu, QString iText,
 
 void MainWindow::createMenu()
 {
+    // Decide whether to show the toolbar
+    QSettings qs(ORGANIZATION, PACKAGE_NAME);
+    bool showToolBar = qs.value(PrefsDialog::kINTF_SHOW_TOOLBAR, true).toBool();
+    m_ui.toolBar->setVisible(showToolBar);
+
     QMenu *menuFile = new QMenu(m_ui.menubar);
     m_ui.menubar->addAction(menuFile->menuAction());
     menuFile->setTitle(_q("&Game"));
@@ -330,10 +389,38 @@ void MainWindow::createMenu()
                   _q("Save the current game"), SLOT(onGameSaveAs()));
     menuFile->addSeparator();
     m_actionGamePrint = addMenuAction(menuFile, _q("&Print..."), _q("Ctrl+P"),
-                  _q("Print the current game"), SLOT(onGamePrint()));
+                  _q("Print the current game"), SLOT(onGamePrint()),
+                  false, QIcon(":/images/printer.png"));
     menuFile->addSeparator();
     addMenuAction(menuFile, _q("&Quit"), _q("Ctrl+Q"),
-                  _q("Quit Eliot"), SLOT(close()));
+                  _q("Quit Eliot"), SLOT(close()),
+                  false, QIcon(":/images/quit_16px.png"));
+
+    QMenu *menuHistory = new QMenu(m_ui.menubar);
+    m_ui.menubar->addAction(menuHistory->menuAction());
+    menuHistory->setTitle(_q("&History"));
+    m_actionHistoryFirstTurn = addMenuAction(menuHistory, _q("&First turn"), _q("Ctrl+Home"),
+                  _q("Go to the first turn of the game"), SLOT(onHistoryFirstTurn()),
+                  false, QIcon(":/images/go-first.png"));
+    m_actionHistoryPrevTurn = addMenuAction(menuHistory, _q("&Previous turn"), _q("Ctrl+Left"),
+                  _q("Go to the previous turn of the game"), SLOT(onHistoryPrevTurn()),
+                  false, QIcon(":/images/go-previous.png"));
+    m_actionHistoryNextTurn = addMenuAction(menuHistory, _q("&Next turn"), _q("Ctrl+Right"),
+                  _q("Go to the next turn of the game"), SLOT(onHistoryNextTurn()),
+                  false, QIcon(":/images/go-next.png"));
+    m_actionHistoryLastTurn = addMenuAction(menuHistory, _q("&Last turn"), _q("Ctrl+End"),
+                  _q("Go to the last turn of the game"), SLOT(onHistoryLastTurn()),
+                  false, QIcon(":/images/go-last.png"));
+    m_actionHistoryReplayTurn = addMenuAction(menuHistory, _q("&Replay turn"), _q("Ctrl+R"),
+                  _q("Play the game from the current position, "
+                     "replacing what was really played"), SLOT(onHistoryReplayTurn()),
+                  false, QIcon(":/images/go-jump.png"));
+    // Add actions to the toolbar
+    m_ui.toolBar->addAction(m_actionHistoryFirstTurn);
+    m_ui.toolBar->addAction(m_actionHistoryPrevTurn);
+    m_ui.toolBar->addAction(m_actionHistoryNextTurn);
+    m_ui.toolBar->addAction(m_actionHistoryLastTurn);
+    m_ui.toolBar->addAction(m_actionHistoryReplayTurn);
 
     QMenu *menuSettings = new QMenu(m_ui.menubar);
     m_ui.menubar->addAction(menuSettings->menuAction());
@@ -341,25 +428,31 @@ void MainWindow::createMenu()
     addMenuAction(menuSettings, _q("&Choose dictionary..."), _q("Ctrl+C"),
                   _q("Select a new dictionary"), SLOT(onSettingsChooseDic()));
     addMenuAction(menuSettings, _q("&Preferences..."), _q("Ctrl+F"),
-                  _q("Edit the preferences"), SLOT(onSettingsPreferences()));
+                  _q("Edit the preferences"), SLOT(onSettingsPreferences()),
+                  false, QIcon(":/images/preferences.png"));
 
     QMenu *menuWindows = new QMenu(m_ui.menubar);
     m_ui.menubar->addAction(menuWindows->menuAction());
     menuWindows->setTitle(_q("&Windows"));
+    m_actionWindowsToolbar = addMenuAction(menuWindows, _q("&Toolbar"), _q("Ctrl+T"),
+                  _q("Show/hide the toolbar"), SLOT(onWindowsToolbar()), true);
+    m_actionWindowsToolbar->setChecked(showToolBar);
     m_actionWindowsBag = addMenuAction(menuWindows, _q("&Bag"), _q("Ctrl+B"),
                   _q("Show/hide the remaining tiles in the bag"), SLOT(onWindowsBag()), true);
     m_actionWindowsBoard = addMenuAction(menuWindows, _q("&External board"), _q("Ctrl+E"),
                   _q("Show/hide the external board"), SLOT(onWindowsBoard()), true);
     m_actionWindowsHistory = addMenuAction(menuWindows, _q("&History"), _q("Ctrl+H"),
-                  _q("Show/hide the game history"), SLOT(onWindowsHistory()), true);
+                  _q("Show/hide the game history"), SLOT(onWindowsHistory()),
+                  true, QIcon(":/images/playlist_16px.png"));
     m_actionWindowsDicTools = addMenuAction(menuWindows, _q("&Dictionary tools"), _q("Ctrl+D"),
                   _q("Show/hide the dictionary tools"), SLOT(onWindowsDicTools()), true);
 
     QMenu *menuHelp = new QMenu(m_ui.menubar);
     m_ui.menubar->addAction(menuHelp->menuAction());
     menuHelp->setTitle(_q("&Help"));
-    addMenuAction(menuHelp, _q("&About..."), _q("Ctrl+A"),
-                  _q("About Eliot"), SLOT(onHelpAbout()));
+    addMenuAction(menuHelp, _q("&About..."), QString(""),
+                  _q("About Eliot"), SLOT(onHelpAbout()),
+                  false, QIcon(":/images/info_16px.png"));
 }
 
 
@@ -408,12 +501,13 @@ void MainWindow::onGameLoad()
     if (fileName != "")
     {
         destroyCurrentGame();
-        m_game = GameFactory::Instance()->load(qtl(fileName), *m_dic);
-        if (m_game == NULL)
+        Game *tmpGame = GameFactory::Instance()->load(qtl(fileName), *m_dic);
+        if (tmpGame == NULL)
         {
             displayErrorMsg(_q("Error while loading the game"));
             return;
         }
+        m_game = new PublicGame(*tmpGame);
         m_ui.groupBoxPlayers->show();
         emit gameChangedNonConst(m_game);
         emit gameChanged(m_game);
@@ -589,8 +683,8 @@ void MainWindow::onSettingsPreferences()
     if (m_prefsDialog == NULL)
     {
         m_prefsDialog = new PrefsDialog(this);
-        QObject::connect(m_prefsDialog, SIGNAL(gameUpdated()),
-                         this, SIGNAL(gameUpdated()));
+        QObject::connect(m_prefsDialog, SIGNAL(prefsUpdated()),
+                         this, SLOT(prefsUpdated()));
     }
     m_prefsDialog->exec();
 }
@@ -600,10 +694,13 @@ void MainWindow::onSettingsChooseDic()
 {
     if (m_game)
     {
-        int res = QMessageBox::question(this, _q("Stop current game?"),
-                                        _q("Loading a dictionary will stop the current game. Do you want to continue?"),
-                                        QMessageBox::Yes | QMessageBox::Default,
-                                        QMessageBox::No | QMessageBox::Escape);
+        QString msg = _q("Loading a dictionary will stop the current game.");
+        QMessageBox confirmationBox(QMessageBox::Question, _q("Eliot"), msg,
+                                    QMessageBox::Yes | QMessageBox::No, this);
+        confirmationBox.setInformativeText(_q("Do you want to continue?"));
+        confirmationBox.setDefaultButton(QMessageBox::Yes);
+        confirmationBox.setEscapeButton(QMessageBox::No);
+        int res = confirmationBox.exec();
         if (res == QMessageBox::No)
             return;
     }
@@ -624,7 +721,6 @@ void MainWindow::onSettingsChooseDic()
 
             // Save the location of the dictionary in the preferences
             QSettings qs(ORGANIZATION, PACKAGE_NAME);
-            QString dicPath = qs.value(PrefsDialog::kINTF_DIC_PATH, "").toString();
             qs.setValue(PrefsDialog::kINTF_DIC_PATH, fileName);
         }
         catch (std::exception &e)
@@ -635,6 +731,17 @@ void MainWindow::onSettingsChooseDic()
 }
 
 
+void MainWindow::onWindowsToolbar()
+{
+    if (m_ui.toolBar->isVisible())
+        m_ui.toolBar->hide();
+    else
+        m_ui.toolBar->show();
+    QSettings qs(ORGANIZATION, PACKAGE_NAME);
+    qs.setValue(PrefsDialog::kINTF_SHOW_TOOLBAR, m_ui.toolBar->isVisible());
+}
+
+
 void MainWindow::onWindowsBag()
 {
     if (m_bagWindow == NULL)
@@ -644,8 +751,8 @@ void MainWindow::onWindowsBag()
         bag->setGame(m_game);
         m_bagWindow = new AuxWindow(*bag, _q("Bag"), "BagWindow",
                                     m_actionWindowsBag);
-        QObject::connect(this, SIGNAL(gameChanged(const Game*)),
-                         bag, SLOT(setGame(const Game*)));
+        QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                         bag, SLOT(setGame(const PublicGame*)));
         QObject::connect(this, SIGNAL(gameUpdated()),
                          bag, SLOT(refresh()));
     }
@@ -662,8 +769,8 @@ void MainWindow::onWindowsBoard()
         board->setGame(m_game);
         m_boardWindow = new AuxWindow(*board, _q("Board"), "BoardWindow",
                                       m_actionWindowsBoard);
-        QObject::connect(this, SIGNAL(gameChanged(const Game*)),
-                         board, SLOT(setGame(const Game*)));
+        QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                         board, SLOT(setGame(const PublicGame*)));
         QObject::connect(this, SIGNAL(gameUpdated()),
                          board, SLOT(refresh()));
     }
@@ -680,8 +787,8 @@ void MainWindow::onWindowsHistory()
         history->setGame(m_game);
         m_historyWindow = new AuxWindow(*history, _q("History"), "HistoryWindow",
                                         m_actionWindowsHistory);
-        QObject::connect(this, SIGNAL(gameChanged(const Game*)),
-                         history, SLOT(setGame(const Game*)));
+        QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
+                         history, SLOT(setGame(const PublicGame*)));
         QObject::connect(this, SIGNAL(gameUpdated()),
                          history, SLOT(refresh()));
     }
@@ -699,6 +806,14 @@ void MainWindow::onWindowsDicTools()
                                     m_actionWindowsDicTools);
         QObject::connect(this, SIGNAL(dicChanged(const Dictionary*)),
                          dicTools, SLOT(setDic(const Dictionary*)));
+        // Link the training rack with the "Plus 1" one
+        QSettings qs(ORGANIZATION, PACKAGE_NAME);
+        if (qs.value(PrefsDialog::kINTF_LINK_TRAINING_7P1, false).toBool())
+        {
+            QObject::connect(m_playersWidget,
+                             SIGNAL(trainingRackUpdated(const QString&)),
+                             dicTools, SLOT(setPlus1Rack(const QString&)));
+        }
         // Fake a dictionary selection
         dicTools->setDic(m_dic);
         dicTools->setFocus();
@@ -712,15 +827,96 @@ void MainWindow::onHelpAbout()
     QString msg;
     msg.sprintf("Eliot %s\n\n", VERSION);
     msg += _q( \
-        "Copyright (C) 1999-2008 - Antoine Fraboulet & Olivier Teuliere\n\n" \
+        "Copyright (C) 1999-2009 - Antoine Fraboulet & Olivier Teuliere\n\n" \
         "This program is free software; you can redistribute it and/or " \
         "modify it under the terms of the GNU General Public License as " \
         "published by the Free Software Foundation; either version 2 of " \
         "the License, or (at your option) any later version.");
     // QMessageBox::about() doesn't add the nice information icon, so we create
     // the box manually (not much work...)
-    QMessageBox *aboutBox = new QMessageBox(QMessageBox::Information,
-                                            _q("About Eliot"), msg, QMessageBox::Ok, this);
-    aboutBox->exec();
+    QMessageBox aboutBox(QMessageBox::Information, _q("About Eliot"),
+                         msg, QMessageBox::Ok, this);
+    aboutBox.exec();
+}
+
+
+void MainWindow::onHistoryFirstTurn()
+{
+    if (m_game == NULL)
+        return;
+
+    m_game->firstTurn();
+    emit gameUpdated();
+    unsigned int currTurn = m_game->getCurrTurn();
+    unsigned int nbTurns = m_game->getNbTurns();
+    displayInfoMsg(_q("Turn %1/%2").arg(currTurn).arg(nbTurns));
+}
+
+
+void MainWindow::onHistoryPrevTurn()
+{
+    if (m_game == NULL)
+        return;
+
+    m_game->prevTurn();
+    emit gameUpdated();
+    unsigned int currTurn = m_game->getCurrTurn();
+    unsigned int nbTurns = m_game->getNbTurns();
+    displayInfoMsg(_q("Turn %1/%2").arg(currTurn).arg(nbTurns));
+}
+
+
+void MainWindow::onHistoryNextTurn()
+{
+    if (m_game == NULL)
+        return;
+
+    m_game->nextTurn();
+    emit gameUpdated();
+    unsigned int currTurn = m_game->getCurrTurn();
+    unsigned int nbTurns = m_game->getNbTurns();
+    displayInfoMsg(_q("Turn %1/%2").arg(currTurn).arg(nbTurns));
+}
+
+
+void MainWindow::onHistoryLastTurn()
+{
+    if (m_game == NULL)
+        return;
+
+    m_game->lastTurn();
+    emit gameUpdated();
+    unsigned int currTurn = m_game->getCurrTurn();
+    unsigned int nbTurns = m_game->getNbTurns();
+    displayInfoMsg(_q("Turn %1/%2").arg(currTurn).arg(nbTurns));
+}
+
+
+void MainWindow::onHistoryReplayTurn()
+{
+    if (m_game == NULL)
+        return;
+
+    QSettings settings(ORGANIZATION, PACKAGE_NAME);
+    bool warn = settings.value(PrefsDialog::kINTF_WARN_REPLAY_TURN, true).toBool();
+    if (warn) {
+        // Ask for a confirmation, because this may lead to data loss
+        QString msg = _q("Replaying this turn will modify the game history "
+                         "by deleting the turns after the displayed one (i.e. "
+                         "turns \"in the future\").");
+        QMessageBox confirmationBox(QMessageBox::Question, _q("Eliot"), msg,
+                                    QMessageBox::Yes | QMessageBox::No, this);
+        confirmationBox.setInformativeText(_q("Do you want to continue?"));
+        confirmationBox.setDefaultButton(QMessageBox::Yes);
+        confirmationBox.setEscapeButton(QMessageBox::No);
+        int ret = confirmationBox.exec();
+        if (ret != QMessageBox::Yes)
+            return;
+    }
+
+    unsigned int currTurn = m_game->getCurrTurn();
+    m_game->clearFuture();
+    emit gameUpdated();
+    displayInfoMsg(_q("Replaying from turn %1").arg(currTurn));
 }
 
