@@ -25,8 +25,7 @@
 #include "ui/training_widget.ui.h"
 
 
-class Game;
-class Training;
+class PublicGame;
 class RackValidator;
 class QStandardItemModel;
 class QString;
@@ -39,12 +38,14 @@ public:
     explicit TrainingWidget(QWidget *parent = 0);
 
 public slots:
-    void setGame(Game *iGame);
+    void setGame(PublicGame *iGame);
     void refresh();
 
 signals:
     void gameUpdated();
     void notifyProblem(QString iMsg);
+    void notifyInfo(QString iMsg);
+    void rackUpdated(const QString &iRack);
 
 protected:
     /// Define a default size
@@ -54,7 +55,7 @@ private slots:
     void enablePlayButton(const QItemSelection &, const QItemSelection &);
     void showPreview(const QItemSelection &, const QItemSelection &);
 
-    // These slots are automatocally connected
+    // These slots are automatically connected
     void on_lineEditRack_textEdited(const QString &iText);
     void on_pushButtonRack_clicked();
     void on_pushButtonComplement_clicked();
@@ -64,7 +65,7 @@ private slots:
 
 private:
     /// Encapsulated training game, can be NULL
-    Training *m_game;
+    PublicGame *m_game;
 
     /// Model of the search results
     QStandardItemModel *m_model;
